/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/

package com.cesys.cebomsr;

import com.cesys.utils.Loader;

import java.io.IOException;

/**
 * Offers functionality for counter periphery. Instances can be retrieved calling
 * getCounters() of the respective {@link Device} instance.
 *
 * @author Thomas Hoppe
 */
public final class Counter implements Input {
	/**
	 * The possible trigger configuration values.
	 */
	public enum CounterConfig {
		RisingEdge(SystemConnector.CeboMsrCounterConfigRisingEdge),
		FallingEdge(SystemConnector.CeboMsrCounterConfigFallingEdge),
		Alternating(SystemConnector.CeboMsrCounterConfigAlternating);

		private final int _counterConfigId;
		private CounterConfig(int counterConfigId) {
			_counterConfigId = counterConfigId;
		}

		static CounterConfig getById(int id) {
			switch (id) {
			case SystemConnector.CeboMsrCounterConfigRisingEdge: return RisingEdge;
			case SystemConnector.CeboMsrCounterConfigFallingEdge: return FallingEdge;
			case SystemConnector.CeboMsrCounterConfigAlternating: return Alternating;
			}
			throw new IllegalArgumentException();
		}
	}

	private final Device _device;
	private final int _interfaceId;
	private final int _index;
	private final String _name;

	int getInterfaceId() {
		return _interfaceId;
	}

	int getIndex() {
		return _index;
	}

	Counter(Device device, int interfaceId, int index) throws IOException {
		_device = device;
		_interfaceId = interfaceId;
		_index = index;
		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		Loader.checkError(SystemConnector.CeboMsrGetPeripheralName(device.getHandle(), interfaceId, tmp, SystemConnector.MaxIdentLength));
		_name = Utils.convertByteArrayToString(tmp);
	}

	/**
	 * Reset counter to 0.
	 */
	public void reset() throws IOException {
		Loader.checkError(SystemConnector.CeboMsrResetCounter(_device.getHandle(), getInterfaceId()));
	}

	/**
	 * Enable or disable counter.
	 * @param enable State to set.
	 */
	public void setEnabled(boolean enable) throws IOException {
		Loader.checkError(SystemConnector.CeboMsrSetCounterEnable(_device.getHandle(), getInterfaceId(), enable ? 1 : 0));
	}

	/**
	 * Get current counter state.
	 */
	public boolean isEnabled() throws IOException {
		int[] r = new int[1];
		Loader.checkError(SystemConnector.CeboMsrGetCounterEnable(_device.getHandle(), getInterfaceId(), r));
		return r[0] != 0;
	}

	/**
	 * Define behavior of the counter.
	 * @param counterConfig Use one of the values from Counter::CounterConfig.
	 */
	public void setConfig(CounterConfig counterConfig) throws IOException {
		Loader.checkError(SystemConnector.CeboMsrSetCounterConfig(_device.getHandle(), getInterfaceId(), counterConfig._counterConfigId));
	}

	/**
	 * Request configuration of the counter.
	 * @return One of the values from Counter::CounterConfig.
	 */
	public CounterConfig getConfig() throws IOException {
		int[] r = new int[1];
		Loader.checkError(SystemConnector.CeboMsrGetCounterConfig(_device.getHandle(), getInterfaceId(), r));
		return CounterConfig.getById(r[0]);
	}

	/**
	 * Read current counter value.
	 */
	public long read() throws IOException {
		long[] r = new long[1];
		Loader.checkError(SystemConnector.CeboMsrReadCounter(_device.getHandle(), getInterfaceId(), r));
		return r[0];
	}

	/**
	 * Returns name of the component.
	 */
	public String getName() {
		return _name;
	}

	public String toString() {
		return "Counter" + _index;
	}
}
