/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/

package com.cesys.cebomsr;

import com.cesys.utils.Loader;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Offers functionality to work with digital ports. Direct construction
 * is not possible. Instances can be retrieved calling getDigitalPorts() of
 * the respective {@link Device} instance.
 *
 * @author Thomas Hoppe
 */
public final class DigitalPort implements Input, Output {
	private final Device _device;
	private final int _interfaceId;
	private final int _index;
	private final int _ioCount;
	private final String _name;
	private final List<String> _ioNames = new ArrayList<String>();

	int getInterfaceId() {
		return _interfaceId;
	}

	int getIndex() {
		return _index;
	}

	DigitalPort(Device device, int interfaceId, int index) throws IOException {
		_device = device;
		_interfaceId = interfaceId;
		_index = index;
		int[] r = new int[1];
		Loader.checkError(SystemConnector.CeboMsrGetDigitalPortIoCount(device.getHandle(), getInterfaceId(), r));
		_ioCount = r[0];
		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		Loader.checkError(SystemConnector.CeboMsrGetPeripheralName(device.getHandle(), interfaceId, tmp, SystemConnector.MaxIdentLength));
		_name = Utils.convertByteArrayToString(tmp);
		for (int i = 0; i < _ioCount; ++i) {
			Loader.checkError(SystemConnector.CeboMsrGetDigitalPortIoName(device.getHandle(), interfaceId, i, tmp, SystemConnector.MaxIdentLength));
			_ioNames.add(Utils.convertByteArrayToString(tmp));
		}
	}

	/**
	 * Set output mask. The default mask is 0.
	 * @param mask Mask to use, bits that have state '1' specify output direction.
	 */
	public void setOutputEnableMask(int mask) throws IOException {
		Loader.checkError(SystemConnector.CeboMsrSetDigitalPortOutputEnableMask(_device.getHandle(), getInterfaceId(), mask));
	}

	/**
	 * Return count of I/O's for the specified port.
	 */
	public int getIoCount() {
		return _ioCount;
	}

	/**
	 * Read value currently active on the port.
	 */
	public int read() throws IOException {
		int r[] = new int[1];
		Loader.checkError(SystemConnector.CeboMsrReadDigitalPort(_device.getHandle(), getInterfaceId(), r));
		return r[0];
	}

	/**
	 * Set output I/O's. Only the bits that have been defined as output
	 * using setOutputEnableMask() are modified.
	 * @param value Value to set on this port.
	 */
	public void write(int value) throws IOException {
		Loader.checkError(SystemConnector.CeboMsrWriteDigitalPort(_device.getHandle(), getInterfaceId(), value));
	}

	/**
	 * Returns name of the component.
	 */
	public String getName() {
		return _name;
	}

	/**
	 * Return name of the I/O as specified by io.
	 * @param io I/O index in range (0 &lt;= io &lt; getIoCount()).
	 */
	public String getIoName(int io) {
		return _ioNames.get(io);
	}

	public String toString() {
		return "DigitalPort" + _index;
	}
}
